#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <regex.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "euterpe.h"
#include "media.h"

mediaFile *mediaStart, *mediaEnd;

void media_add(char *path, char *filename) {
	mediaFile *newmedia = malloc(sizeof(mediaFile));

	newmedia->path = malloc(strlen(path) + 1);
	strcpy(newmedia->path, path);
	newmedia->filename = malloc(strlen(filename) + 1);
	strcpy(newmedia->filename, filename);
	newmedia->next = NULL;

	if (mediaEnd) {
		mediaEnd->next = newmedia;
	} else {
		mediaStart = newmedia;
	}

	mediaEnd = newmedia;
}

void media_clear() {
	mediaFile *curmedia = mediaStart;
	mediaFile *next;

	mediaStart = NULL;
	mediaEnd = NULL;

	while (curmedia) {
		next = curmedia->next;

		free(curmedia->path);
		free(curmedia->filename);
		free(curmedia);

		curmedia = next;
	}
}

void media_search(char *query) {
	mediaFile *curmedia = mediaStart;
	regex_t preg;
	int i, len, len2, first = 1;
	char *regstr, *actionstr;

	if (!actionFirst || !actionRest) {
		return;
	}

	/* Count up the number of whitespace chars to account for
		 the added length of '[[:space:]]*' in the regstr */
	len = len2 = strlen(query);
	for (i = 0; i < len; i++) {
		if (isspace(query[i])) {
			len2 += strlen(WS_REGEXP);
		}
	}

	regstr = malloc(len2 + 2 * strlen(ANY_REGEXP) + 1);
	regstr[0] = '\0';
	strcat(regstr, ANY_REGEXP); 
	for (i = 0; i < len; i++) {
		if (isspace(query[i])) {
			if (i != len - 1) {
				strcat(regstr, WS_REGEXP); 
			}
		} else {
			strncat(regstr, &query[i], 1);
		}
	}
	strcat(regstr, ANY_REGEXP); 

	//printf("'%s'\n", regstr);

	if (regcomp(&preg, regstr, REG_ICASE | REG_NOSUB)) {
		free(regstr);
		return;
	}

	while (curmedia) {
		if (regexec(&preg, curmedia->filename, 0, NULL, 0) == 0) {
			if (first) {
				if (actionInit) {
					// Only execute actionInit if there are hits
					system(actionInit);
				}

				actionstr = malloc(strlen(actionFirst) + 2 + strlen(curmedia->path) +
													 1 + strlen(curmedia->filename) + 2);
				sprintf(actionstr, 
								"%s \"%s/%s\"", 
								actionFirst, 
								curmedia->path, 
								curmedia->filename);
				first = 0;
			} else {
				actionstr = malloc(strlen(actionRest) + 2 + strlen(curmedia->path) +
													 1 + strlen(curmedia->filename) + 2);
				sprintf(actionstr, 
								"%s \"%s/%s\"", 
								actionRest, 
								curmedia->path, 
								curmedia->filename);
			}
			system(actionstr);
			free(actionstr);
		}

		curmedia = curmedia->next;
	}

	if (!first && actionFini) {
		// Only execute actionFini if there were hits
		system(actionFini);
	}

	regfree(&preg);
	free(regstr);
}

void media_load_files(DIR *archive, char *path) {
	struct dirent *entry;
	struct stat filestat;
	
	while ((entry = readdir(archive)) != NULL) {
		/* lstat to not get caught in symlink loops */
		if (lstat(entry->d_name, &filestat) != -1) {
			if  (entry->d_name[0] == '.') {
				/* Do nothing */
			} else if (S_ISDIR(filestat.st_mode)) {
				/* recurse on subdirectories */
				media_load_directory(entry->d_name);
			} else {
				media_add(path, entry->d_name);
			}
		}
	}
}

void media_load_directory(char *path) {
	DIR *archive;
	char buf[256];

	archive = opendir(path);

	if (archive != NULL) {
		chdir(path);
		getcwd(buf, 256);
		media_load_files(archive, buf);
		chdir("..");
		closedir(archive);
	}
}

void media_load_archives() {
	char *path = strdup(archivePath);
	char *subpath;

	media_clear();

	while ((subpath = strsep(&path, ":")) != NULL) {
		media_load_directory(subpath);
	}
}
